/* Copyright (C) 2002 Timo Sirainen */

#include "common.h"
#include "temp-string.h"
#include "commands.h"

extern MailboxSyncCallbacks sync_callbacks;

int cmd_select_full(Client *client, int readonly)
{
	Mailbox *box;
	MailboxStatus status;
	const char *mailbox;

	/* <mailbox> */
	if (!client_read_string_args(client, 1, &mailbox))
		return FALSE;

	if (client->mailbox != NULL)
		client->mailbox->close(client->mailbox);

	client->mailbox = client->storage->open_mailbox(client->storage,
							mailbox, readonly,
							FALSE);
	if (client->mailbox == NULL) {
		client_send_storage_error(client);
		return TRUE;
	}

	box = client->mailbox;
	if (!box->get_status(box, STATUS_MESSAGES | STATUS_RECENT |
			     STATUS_FIRST_UNSEEN_SEQ | STATUS_UIDVALIDITY |
			     STATUS_CUSTOM_FLAGS, &status)) {
		client_send_storage_error(client);
		return TRUE;
	}

	/* set callbacks after STATUS, which might otherwise try calling
	   some of them */
	box->set_sync_callbacks(box, &sync_callbacks, client);

	client_send_mailbox_flags(client, box, status.custom_flags,
				  status.custom_flags_count);

	client_send_line(client,
		t_strdup_printf("* %u EXISTS", status.messages));
	client_send_line(client,
		t_strdup_printf("* %u RECENT", status.recent));

	if (status.first_unseen_seq != 0) {
		client_send_line(client,
			t_strdup_printf("* OK [UNSEEN %u] First unseen.",
					status.first_unseen_seq));
	}

	client_send_line(client,
		t_strdup_printf("* OK [UIDVALIDITY %u] UIDs valid",
				status.uidvalidity));

	if (status.diskspace_full) {
		client_send_line(client, "* OK [ALERT] "
				 "Disk space is full, delete some messages.");
	}

	client_send_tagline(client, box->readonly ?
			    "OK [READ-ONLY] Select completed." :
			    "OK [READ-WRITE] Select completed.");

	return TRUE;
}

int cmd_select(Client *client)
{
	return cmd_select_full(client, FALSE);
}
